-----------------------------------------------------------------------------------------------
-- Client Lua Script for ReCrisis
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ReCrisis Module Definition
-----------------------------------------------------------------------------------------------
local ReCrisis = {} 
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ReCrisis:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ReCrisis:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ReCrisis OnLoad
-----------------------------------------------------------------------------------------------
function ReCrisis:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ReCrisis.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ReCrisis OnDocLoaded
-----------------------------------------------------------------------------------------------
function ReCrisis:OnDocLoaded()
	--Load spriterinos
	Apollo.LoadSprites("spr_recri.xml");
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "frm_indicator", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		-- In order to avoid using timers, we track GR casts by the player using OnCombatLogHeal
		Apollo.RegisterEventHandler("CombatLogHeal","OnCombatLogHeal", self)
		Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
		self.CrisisTimer = ApolloTimer.Create(0.2, false, "OnCrisisTimer", self)
		
		-- Do additional Addon initialization here
		self.indicator = self.wndMain:FindChild("lbl_indicator");
		self.currentCrisis = 1;
		self.isCombat = false;
		self.CrisisTimer:Stop();
	end
end

-----------------------------------------------------------------------------------------------
-- ReCrisis Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function ReCrisis:OnCombatLogHeal(tEventArgs)
	local unitCaster = tEventArgs.unitCaster;
	local spellName = tEventArgs.splCallingSpell:GetName();

	if unitCaster == GameLib.GetPlayerUnit() and spellName == "Crisis Wave" then
		self.CrisisTimer:Start();	
	end
end

function ReCrisis:OnCrisisTimer()
	--This is where our functionality goes because hurr 3 Crisis ray beams
	--Count the Crisis rays
	if self.isCombat then
		if self.currentCrisis < 3 then
			self.currentCrisis = self.currentCrisis + 1;
			self.wndMain:Close();
		else
			self.currentCrisis = 1;
			self.wndMain:Close();
		end
		
		if self.currentCrisis == 3 then
			self.wndMain:Invoke();
			Sound.PlayFile("codecopen.wav");
		end
	end
end

--Fired when combat entered
function ReCrisis:OnEnteredCombat(u, isCombat)
	if u == GameLib.GetPlayerUnit() and not isCombat then
		self.isCombat = false;
		self.currentCrisis = 1;
		self.wndMain:Close();
	end
	
	if u == GameLib.GetPlayerUnit() and isCombat then
		if not self:isActive("Crisis Wave") then
			return
		end
		
		self.isCombat = true;
	end
end

--Check for spell on LAS
function ReCrisis:isActive(spell)
	local _spell = ReCrisis:GetSpellByName(spell);
	local las = ActionSetLib.GetCurrentActionSet()
	
	if las then
		for i,v in ipairs(las) do
			if _spell[1]["nId"] == las[i] then
				return true
			end
		end
	end
	
	return false;
end

--Gets a spell from the name
function ReCrisis:GetSpellByName(spellName)
	local matches = {}
	local abilities = AbilityBook.GetAbilitiesList()
	if abilities ~= nil then
		for _, ability in pairs(abilities) do
			if ability.strName == spellName then
				table.insert(matches, ability)
			end
		end
	end
	return matches
end

--Save loc of the window
function ReCrisis:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "recrisierino"})
end


-----------------------------------------------------------------------------------------
-- ReCrisisForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function ReCrisis:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function ReCrisis:OnCancel()
	self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- ReCrisis Instance
-----------------------------------------------------------------------------------------------
local ReCrisisInst = ReCrisis:new()
ReCrisisInst:Init()
